/**
@author   Cody Johnson
@email    cody.ele@noritake.com (Feel free to ask questions)
@website  noritake-elec.com
@version  v1.1 (ME-N58-1)
@ide      IAR
@brief    GT-CP command library for STM32F4xx and Noritake GT-CP series modules

@vertatim
Provide the GT-CP commands between STM32F4Discovery and Noritake GT-CP series modules.
@endverbatim
*/

#include "GTCP.h"
#include "GTCP_I2C.h"
#include "GTCP_SPI.h"
#include "GTCP_USART.h"
#include <stddef.h>

//Interface definitions
#define ASYNCH 0
#define SPI 1
#define I2C 2

//Display variables.
unsigned WIDTH;
unsigned HEIGHT;
unsigned LINES; // The number of character lines present on the display.
int interface;

/**
Initializes the Noritake GTCP series module.

@param  width   The width of the display in dots.
@param  height  The height of the display in dots.
@return none
*/
void GTCP_Init(unsigned width, unsigned height){
        WIDTH = width;
        HEIGHT = height;
        LINES = height / 8;
        
	GTCP_print(0x1b);
	GTCP_print(0x40);
}

/**
The universal print function for this library.
Prints bytes based on the interface value set by each interface's initialization function.

@param  data The data to be sent.
@return none
*/
void GTCP_print(char data){
  switch(interface){
  case ASYNCH:
    writeUART(data);
    break;
  case SPI:
    writeSPI(data);
    break;
  case I2C:
    GTCP_I2C_Write(data);
    break;
  default:
    break;
  }
}

/**
Performs a backspace on the GTCP display.

@return none
*/
void GTCP_back(){
  GTCP_print(0x08);
}

/**
Moves the cursor forward one position on the GTCP display.

@return none
*/
void GTCP_forward(){
  GTCP_print(0x09);
}

/**
Performs a line feed on the GTCP display.

@return none
*/
void GTCP_lineFeed(){
  GTCP_print(0x0a);
}

/**
Brings the cursor to its home position (top left) on the GTCP display.

@return none
*/
void GTCP_home(){
  GTCP_print(0x0b);
}

/**
Performs a carriage return on the GTCP display.

@return none
*/
void GTCP_carriageReturn(){
  GTCP_print(0x0d);
}

/**
Performs a carriage return and line feed on the GTCP display.

@return none
*/
void GTCP_crlf(){
  //Carriage Return
  GTCP_print(0x0d);
  //Line Feed
  GTCP_print(0x0a);
}

/**
Sends an x and y coordinate to the GTCP display.

@param  x       The desired x coordinate.
@param  y       The desired y coordinate.
@return none
*/
void GTCP_XY(unsigned x, unsigned y){
	y /= 8;
  GTCP_print(x);
  GTCP_print(x >> 8);
  GTCP_print(y);
  GTCP_print(y >> 8);
}

/**
Sends an x and y coordinate to the GTCP display.

@param  x       The desired x coordinate.
@param  y       The desired y coordinate.
@return none
*/
void GTCP_XY1(unsigned x, unsigned y){
  GTCP_print(x);
  GTCP_print(x >> 8);
  GTCP_print(y);
  GTCP_print(y >> 8);
}

/**


@return none
*/
void GTCP_usCommand(){
  GTCP_print(0x1f);
  GTCP_print(0x28);
}

/**
Moves the cursor to a specific location on the GTCP module.

@param  x       The desired x coordinate.
@param  y       The desired y coordinate.
@return none
*/
void GTCP_setCursor(unsigned x, unsigned y){
  GTCP_print(0x1f);
  GTCP_print('$');
  GTCP_XY1(x, y);
}

/**
Clears the screen of the GTCP module.

@return none
*/
void GTCP_clearScreen(){
  GTCP_print(0x0c);
}

/**
Clear the current line. Cursor will return to left end.

@return none
*/
void GTCP_lineClear(){
  GTCP_print(0x18);
}

/**
Clear the current line. Cursor will move to right end.

@return none
*/
void GTCP_lineEndClear(){
  GTCP_print(0x19);
}

/**
Turns the cursor on.

@return none
*/
void GTCP_cursorOn(){
  GTCP_print(0x1f);
  GTCP_print('C');
  GTCP_print(0x01);
}

/**
Turns the cursor off.

@return none
*/
void GTCP_cursorOff(){
  GTCP_print(0x1f);
  GTCP_print('C');
  GTCP_print(0x00);
}

/**
Resets the GTCP series module.

@return none
*/
void GTCP_reset(){
	//TODO
}

void GTCP_8by16DotMode(){
  GTCP_usCommand();
  GTCP_print(0x67);
  GTCP_print(0x01);
  GTCP_print(0x02);
}

/**
Select how display actions will affect the module.
Display screen mode: Display action is valid within the Display or hidden area.
All screen mode: Display action is valid over the entire display memory.
<ul>
<li> mode = 0x00: Display screen mode
<li> mode = 0x01: All screen mode
<li> Default: mode = 0x00 or memory SW setting
</ul>

@param  mode The desired write screen mode.
@return none
*/
void GTCP_writeScreenMode(uint8_t mode){
  GTCP_print(0x1f);
  GTCP_print(0x28);
  GTCP_print(0x77);
  GTCP_print(0x10);
  GTCP_print(mode);
}

/**
Select how background pixels are written to the display memory.

<ul>
<li> mode = 0x10: Normal display write (Background pixels are written to display memory)
<li> mode = 0x11: Thru write (Background pixels are not written to display memory)
<li> Default: mode = 0x10
</ul>

@param  mode The desired mixture display mode
@return none
*/
void GTCP_writeMixtureDisplayMode(uint8_t mode){
  GTCP_print(0x1f);
  GTCP_print(0x77);
  GTCP_print(mode);
}

/**
Enables the use of multiple-byte characters.

@param  enable  Enable or disable multiple byte characters.
@return none
*/
void GTCP_useMultiByteChars(uint8_t enable){
  GTCP_usCommand();
  GTCP_print('g');
  GTCP_print(0x02);
  GTCP_print(enable);
}

/**
Sets the multiple byte character set to be used.

Code for each character set:
<ul>
<li> 0x00 - Japanese
<li> 0x01 - Korean
<li> 0x02 - Simplified Chinese
<li> 0x03 - Traditional Chinese
</ul>

@param  code    The character set code to be used.
@return none
*/
void GTCP_setMultiByteCharSet(uint8_t code){
  GTCP_usCommand();
  GTCP_print('g');
  GTCP_print(0x0f);
  GTCP_print(code);
}

/**
Set the desired 1-byte character font size.
<ul>
<li> size = 0x00: Outline font
<li> size = 0x01: 6x8 font
<li> size = 0x02: 8x16 font
<li> size = 0x03: 12x24 font
<li> size = 0x04: 16x32 font
</ul>

@param  size The desired character size.
@return none
*/
void GTCP_setFontSize(uint8_t size){
  GTCP_usCommand();
  GTCP_print(0x67);
  GTCP_print(0x01);
  GTCP_print(size);
}

/**
Set the outline font type.
This needs to be set in order to use outline fonts.
<ul>
<li> fontType = 0x00: Japanese
<li> fontType = 0x01: Korean
<li> fontType = 0x02: Simplified Chinese
<li> fontType = 0x03: Traditional Chinese
<li> fontType = 0x80: None (no outline font selected)
<li> fontType = 0xff: User-supplied font file
<li> Default: fontType = 0x80
</ul>

@param  fontType The desired outline font type.
@return none
*/
void GTCP_setOutlineFontType(uint8_t fontType){
  GTCP_usCommand();
  GTCP_print(0x67);
  GTCP_print(0x08);
  GTCP_print(fontType);
}

/**
Enables or disables the use of custom characters.

@param  enable  Enable of disable custom characters.
@return none
*/
void GTCP_useCustomChars(uint8_t enable){
  GTCP_print(0x1b);
  GTCP_print('%');
  GTCP_print(enable);
}

/**
Helper function for GTCP_defineCustomChar.

@param src The source data array to be used.
@param col The column of pixels to get.
@return uint8_t The desired column.
*/
uint8_t GTCP_getColumn(uint8_t *src, int col){
    uint8_t out = 0;
    
    for(int i = 0; i < 8; i++){
		if(src[i] & (1 << (4 - col))){
			out += 1 << (7 - 1);
		}
    }
    return out;
}

/**
code starts at 0x20 and ends at 0x27
<ul>
<li> format = 0x01: 6x8 pixel character
<li> format = 0x02: 8x16 pixel character
<li> format = 0x03: 12x24 pixel character
<li> format = 0x04: 16x32 pixel character
</ul>

To print custom character, write the code of the character
to the display after using this function.

Data format for custom characters:
6x8 character
<ul>
<li> 0b00000000
<li> 0b00000000
<li> 0b00000000
<li> 0b00000000
<li> 0b00000000
<li> 0b00000000
</ul>

8x16 character
<ul>
<li> 0b00000000, 0b00000000
<li> 0b00000000, 0b00000000
<li> 0b00000000, 0b00000000
<li> 0b00000000, 0b00000000
<li> 0b00000000, 0b00000000
<li> 0b00000000, 0b00000000
<li> 0b00000000, 0b00000000
<li> 0b00000000, 0b00000000
</ul>

12x24 character
<ul>
<li> 0b00000000, 0b00000000, 0b00000000
<li> 0b00000000, 0b00000000, 0b00000000
<li> 0b00000000, 0b00000000, 0b00000000
<li> 0b00000000, 0b00000000, 0b00000000
<li> 0b00000000, 0b00000000, 0b00000000
<li> 0b00000000, 0b00000000, 0b00000000
<li> 0b00000000, 0b00000000, 0b00000000
<li> 0b00000000, 0b00000000, 0b00000000
<li> 0b00000000, 0b00000000, 0b00000000
<li> 0b00000000, 0b00000000, 0b00000000
<li> 0b00000000, 0b00000000, 0b00000000
<li> 0b00000000, 0b00000000, 0b00000000
</ul>

16x32 character
<ul>
<li> 0b00000000, 0b00000000, 0b00000000, 0b00000000
<li> 0b00000000, 0b00000000, 0b00000000, 0b00000000
<li> 0b00000000, 0b00000000, 0b00000000, 0b00000000
<li> 0b00000000, 0b00000000, 0b00000000, 0b00000000
<li> 0b00000000, 0b00000000, 0b00000000, 0b00000000
<li> 0b00000000, 0b00000000, 0b00000000, 0b00000000
<li> 0b00000000, 0b00000000, 0b00000000, 0b00000000
<li> 0b00000000, 0b00000000, 0b00000000, 0b00000000
<li> 0b00000000, 0b00000000, 0b00000000, 0b00000000
<li> 0b00000000, 0b00000000, 0b00000000, 0b00000000
<li> 0b00000000, 0b00000000, 0b00000000, 0b00000000
<li> 0b00000000, 0b00000000, 0b00000000, 0b00000000
<li> 0b00000000, 0b00000000, 0b00000000, 0b00000000
<li> 0b00000000, 0b00000000, 0b00000000, 0b00000000
<li> 0b00000000, 0b00000000, 0b00000000, 0b00000000
<li> 0b00000000, 0b00000000, 0b00000000, 0b00000000
</ul>

<ul>
<li> X = don't care
<li> LSB is the bottom of the character.
<li> MSB is the top of the character.
<li> First byte of data is left most column of the character.
<li> Last byte of data is the right most column of the character.
</ul>

@param   code   The code for the character being defined.
@param   format The format of the character being defined.
@param   *data  The data for the character being defined.
@returns none
*/
void GTCP_defineCustomChar(uint8_t codeStart, uint8_t codeEnd, uint8_t format, uint8_t xDir, uint8_t *data){
  int j = 0;
  
  GTCP_print(0x1b);
  GTCP_print('&');
  GTCP_print(format);
  GTCP_print(codeStart);
  GTCP_print(codeEnd);
  GTCP_print(xDir);

  switch(format){
  case 0x01: //6x8 Pixel Character
    while(xDir != 0){
      GTCP_print(data[j]);
      j++;
      xDir--;
    }
    break;
  case 0x02: //8x16 Pixel Character
    xDir *= 2;
    while(xDir != 0){
      GTCP_print(data[j]);
      j++;
      xDir--;
    }
    break;
  case 0x03: //12x24 Pixel Character
    xDir *= 3;
    while(xDir != 0){
      GTCP_print(data[j]);
      j++;
      xDir--;
    }
    break;
  case 0x04: //16x32 Pixel Character
    xDir *= 4;
    while(xDir != 0){
      GTCP_print(data[j]);
      j++;
      xDir--;
    }
    break;
  }
}

/**
Deletes a previously defined custom character.

@param  code    Address of the custom character to be deleted.
@return none
*/
void GTCP_deleteCustomChar(uint8_t type, uint8_t code){
  GTCP_print(0x1b);
  GTCP_print('?');
  GTCP_print(type);
  GTCP_print(code);
}

/**
Sets the desired ASCII variant to be used.
Also called "International font select" in the GTCP software manual.

Font codes:
<ul>
<li> 0x00 - America
<li> 0x01 - France
<li> 0x02 - Germany
<li> 0x03 - England
<li> 0x04 - Denmark 1
<li> 0x05 - Sweden
<li> 0x06 - Italy
<li> 0x07 - Spain 1
<li> 0x08 - Japan
<li> 0x09 - Norway
<li> 0x0A - Denmark 2
<li> 0x0B - Spain 2
<li> 0x0C - Latin America
<li> 0x0D - Korea
</ul>

@param  code   The code of the ASCII variant to be used.
@return none
*/
void GTCP_setAsciiVariant(uint8_t code){
	if(code < 0x0d){
          GTCP_print(0x1b);
          GTCP_print('R');
          GTCP_print(code);
	}
}

/**
Sets the desired character set.

Character set codes:
<ul>
<li> 0x00 - PC437(USA-Euro std)
<li> 0x01 - Katakana - Japanese
<li> 0x02 - PC850 (Multilingual)
<li> 0x03 - PC860 (Portuguese)
<li> 0x04 - PC863 (Canadian-French)
<li> 0x05 - PC865 (Nordic)
<li> 0x10 - WPC1252 (Latin)
<li> 0x11 - PC866 (Cryllic #2)
<li> 0x12 - PC852 (Latin 2)
<li> 0x13 - PC858 (Eastern European)
</ul>

@param  code   The code for the desired character set.
@return none
*/
void GTCP_setCharSet(uint8_t code){
	if(code < 0x05 || (0x10 <= code && code <= 0x13) || (code == 0xfe) || (code == 0xff)){
          GTCP_print(0x1b);
          GTCP_print('t');
          GTCP_print(code);
	}
}

/**
Sets the scroll mode on the GTCP module.

Scroll modes:
<ul>
<li> 0x01 - Wrapping mode
<li> 0x02 - Vertical scroll mode
<li> 0x03 - Horizontal scroll mode
</ul>
@param  mode   The desired scroll mode.
@return none
*/
void GTCP_setScrollMode(uint8_t mode){
  GTCP_print(0x1f);
  GTCP_print(mode);
}

/**
Sets the horizontal scroll speed on the GTCP series module.

Speed parameter specifics:
<ul>
<li> 0x00 <= speed <= 0x1f
<li> speed = 0x00 : Instantaneous Speed
<li> speed = 0x01 : IntTime / 2 dots
<li> speed = 0x02 - 0x1f : (n-1) * IntTime / dot
</ul>

@param  speed  The desired horizontal scroll speed.
@return none
*/
void GTCP_setHorizScrollSpeed(uint8_t speed){
  GTCP_print(0x1f);
  GTCP_print('s');
  GTCP_print(speed);
}

/**
Turns the display inversion OFF.

@return none
*/
void GTCP_invertOff(){
  GTCP_print(0x1f);
  GTCP_print('r');
  GTCP_print(0x00);
}

/**
Turns the display inversion ON.

@return none
*/
void GTCP_invertOn(){
  GTCP_print(0x1f);
  GTCP_print('r');
  GTCP_print(0x01);
}

/**
Sets the composition mode of the GTCP module.

Composition modes:
<ul>
<li> 0x00 - Normal display write (not mixture display)
<li> 0x01 - OR display write
<li> 0x02 - AND display write
<li> 0x03 - XOR display write
</ul>

@param  mode  The desired composition mode.
@return none
*/
void GTCP_setCompositionMode(uint8_t mode){
  GTCP_print(0x1f);
  GTCP_print('w');
  GTCP_print(mode);
}

/**
Sets the screen brightness of the GTCP module.

Screen brightness levels:
<ul>
<li> 0x01 - 12.5%
<li> 0x02 - 25.0%
<li> 0x03 - 37.5%
<li> 0x04 - 50.0%
<li> 0x05 - 62.5%
<li> 0x06 - 75.0%
<li> 0x07 - 87.5%
<li> 0x08 - 100%
</ul>

@param  level The desired brightness level.
@return none
*/
void GTCP_setScreenBrightness(unsigned level){
	if(level == 0){
		GTCP_displayOff();
	}else if(level <= 100){
		GTCP_displayOn();
                GTCP_print(0x1f);
		GTCP_print('X');
		GTCP_print((level * 10 + 120) / 125);
	}
}

/**
Suspends the program and waits for a user defined amount of time.

@param  wait    The amount of time for the MCU to wait.
@return none
*/
void GTCP_wait(uint8_t wait){
  GTCP_usCommand();
  GTCP_I2C_Write('a');
  GTCP_I2C_Write(0x01);
  GTCP_I2C_Write(wait);
}

/**
Suspends the program and waits for a user defined amount of time.
Wait time = t * IntTime

@param  time The amount of time to wait.
@return none
*/
void GTCP_shortWait(uint8_t time){
  GTCP_usCommand();
  GTCP_print('a');
  GTCP_print(0x02);
  GTCP_print(time);
}

/**
Sets the scroll parameters and scrolls the screen.

@param  x       The amount of pixels for the screen to scroll horizontally.
@param  y       The amount of pixels for the screen to scroll vertically.
@param times    The number of times the screen will scroll?
@param speed    The speed at which scrolling occurs.
@return none
*/
void GTCP_scrollScreen(unsigned x, unsigned y, unsigned times, uint8_t speed){
  GTCP_usCommand();
  GTCP_print('a');
  GTCP_print(0xa0);
  GTCP_XY1(x, y);
  GTCP_print(times);
  GTCP_print(times >> 8);
  GTCP_print(speed);
}

/**
Start a curtain display action on-screen.
<ul>
<li> direction = 0x00: To the right from the left edge.
<li> direction = 0x01: To the left from the right edge.
<li> direction = 0x02: To the left and right separately from the center.
<li> direction = 0x03: To the center from the left and right edge.
</ul>

<ul>
<li> Curtain action speed = 60 * speed * IntTime
<li> 0x00 <= speed <= 0xff
</ul>

<ul>
<li> 0x00 <= red <= 0xff
<li> 0x00 <= green <= 0xff
<li> 0x00 <= blue <= 0xff
</ul>

@param  direction Direction of curtain action.
@param  speed     Curtain action speed.
@param  red       Red brightness
@param  green     Green brightness
@param  blue      Blue brightness
@return none
*/
void GTCP_curtainAction(uint8_t direction, uint8_t speed, uint8_t red, uint8_t green, uint8_t blue){
  GTCP_usCommand();
  GTCP_print(0x61);
  GTCP_print(0xa2);
  GTCP_print(direction);
  GTCP_print(speed);
  GTCP_print(red);
  GTCP_print(green);
  GTCP_print(blue);
}

/**
Start a curtain display action on-screen.
<ul>
<li> direction = 0x00: To the right from the left edge.
<li> direction = 0x01: To the left from the right edge.
<li> direction = 0x02: To the left and right separately from the center.
<li> direction = 0x03: To the center from the left and right edge.
</ul>

<ul>
<li> Curtain action speed = 60 * speed * IntTime
<li> 0x00 <= speed <= 0xff
</ul>

@param  direction Direction of curtain action.
@param  speed     Spring action speed.
@param  x         Display memory X position.
@param  y         Display memory y position.
@return none
*/
void GTCP_springAction(uint8_t direction, uint8_t speed, unsigned x, unsigned y){
  GTCP_usCommand();
  GTCP_print(0x61);
  GTCP_print(0xa3);
  GTCP_print(direction);
  GTCP_print(speed);
  GTCP_XY1(x, y);
}

/**
Start a checkerboard display action.
<ul>
<li> 0x00 <= actionType <= 0x02
<li> actionType = 0x00: Transition pattern 1
<li> actionType = 0x01: Transition pattern 2
<li> actionType = 0x02: Transition pattern 3
<li> 0x00 <= speed <= 0xff
</ul>

@param  direction Direction of curtain action.
@param  speed     Checkerboard action speed.
@param  x         Display memory X position.
@param  y         Display memory y position.
@return none
*/
void GTCP_randomAction(uint8_t actionType, uint8_t speed, unsigned x, unsigned y){
  GTCP_usCommand();
  GTCP_print(0x61);
  GTCP_print(0xa4);
  GTCP_print(actionType);
  GTCP_print(speed);
  GTCP_XY1(x, y);
}

/**
Start a fade-in display action.
<ul>
<li> Fade-in time = speed * approximately 0.5s
</ul>

@param  speed Fade-in action speed.
@param  x     Display memory X position.
@param  y     Display memory Y position.
@return none
*/
void GTCP_fadeInAction(uint8_t speed, unsigned x, unsigned y){
  GTCP_usCommand();
  GTCP_print(0x61);
  GTCP_print(0xa5);
  GTCP_print(speed);
  GTCP_XY1(x, y);
}

/**
Start a fade-out display action.
<ul>
<li> Fade-out time = speed * approximately 0.5s
</ul>

@param  speed Fade-out action speed.
@return none
*/
void GTCP_fadeOutAction(uint8_t speed){
  GTCP_usCommand();
  GTCP_print(0x61);
  GTCP_print(0xa6);
  GTCP_print(speed);
}

/**
Resets the blink settings on the module to all zeros.

@return none
*/
void GTCP_blinkScreenOff(){
  	GTCP_usCommand();
	GTCP_print('a');
	GTCP_print(0x11);
	GTCP_print(0x00);
	GTCP_print(0x00);
	GTCP_print(0x00);
	GTCP_print(0x00);
}
/**
Blinks the GTCP series module screen based on user input parameters.

@param  enable   Enables or disables screen blinking.
                 <ul>
                 <li> 0x00 - Normal display
                 <li> 0x01 - Blink display (alternatively Normal and Blank display)
                 <li> 0x02 - Blink display (alternatively Normal and Reverse display)
                 </ul>
@param  reverse  Enables or disables the reverse blinking pattern.
@param  onTime   The time that the display stays ON during blinking
@param  offTtime The time that the displays stays OFF during blinking.
@return none
*/
void GTCP_blinkScreenOn(uint8_t enable, uint8_t reverse, uint8_t onTime, uint8_t offTime, uint8_t cycles){
  GTCP_usCommand();
  GTCP_print('a');
  GTCP_print(0x11);
  GTCP_print(enable? (reverse? 2: 1): 0);
  GTCP_print(onTime);
  GTCP_print(offTime);
  GTCP_print(cycles);
}

/**
Turns the display OFF.

@return none
*/
void GTCP_displayOff(){
  GTCP_usCommand();
  GTCP_print('a');
  GTCP_print(0x40);
  GTCP_print(0x00);
}

/**
Turns the display ON.

@return none
*/
void GTCP_displayOn(){
  GTCP_usCommand();
  GTCP_print('a');
  GTCP_print(0x40);
  GTCP_print(0x01);
}

/**
Sets the screen saver mode for the GTCP series module.

Screen saver modes:
<ul>
<li> 0x00 - Display power OFF
<li> 0x01 - Display power ON
<li> 0x02 - All dots OFF
<li> 0x03 - All dots ON
<li> 0x04 - Repeat blink display with normal and reverse display
</ul>

@param  mode    The desired screensaver mode. (see ScreenSaver)
@return none
*/
void GTCP_screenSaver(uint8_t mode){
  GTCP_usCommand();
  GTCP_print('a');
  GTCP_print(0x40);
  GTCP_print(mode);
}

/**
Sets the font style for the GTCP series module.

@param  proportional Enable or disable a proportional font style.
@param  evenSpacing  Enable or disable even spacing between characters.
@return none
*/
void GTCP_setFontStyle(uint8_t proportional, uint8_t evenSpacing){
  GTCP_usCommand();
  GTCP_print('g');
  GTCP_print(0x03);
  GTCP_print(proportional * 2 + evenSpacing);
}

/**
Sets the font size for the GTCP series module.

@param  x    X magnification factor
@param  y    Y magnification factor
@return none
*/
void GTCP_setFontMagnification(uint8_t x, uint8_t y, uint8_t tall){
	if(x <= 4 && y <= 4){
          GTCP_usCommand();
          GTCP_print('g');
          GTCP_print(0x40);
          GTCP_print(x);
          GTCP_print(y);
	}
}

/**
Selects the current window.

@param  window  The desired window to select. (0-4)
@return none
*/
void GTCP_selectWindow(uint8_t window){
	if(window <= 4){
          GTCP_print(0x1f);
          GTCP_print(0x28);
          GTCP_print(0x77);
          GTCP_print(0x01);
          GTCP_print(window);
	}
}
/**
Defines a specific window for the GTCP series module.

@param  window  The number of the window to be created. (1-4)
@param  x       The x coordinate of the new window.
@param  y       The y coordinate of the new window.
@param  width   The width of the new window.
@param  height  The height of the new window.
@return none
*/
void GTCP_defineWindow(uint8_t window, unsigned x, unsigned y, unsigned width, unsigned height){
  GTCP_usCommand();
  GTCP_print('w');
  GTCP_print(0x02);
  GTCP_print(window);
  GTCP_print(0x01);
  GTCP_XY1(x, y);
  GTCP_XY1(width, height);
}

/**
Deletes a previously defined window.

@param  window  The number of the window to be deleted. (1-4)
@return none
*/
void GTCP_deleteWindow(uint8_t window){
  GTCP_usCommand();
  GTCP_print('w');
  GTCP_print(0x02);
  GTCP_print(window);
  GTCP_print(0x00);
  GTCP_XY1(0, 0);
  GTCP_XY1(0, 0);
}

/**
Joins all of the screens into one base screen.

@return none
*/
void GTCP_joinScreens(){
  GTCP_usCommand();
  GTCP_print('w');
  GTCP_print(0x10);
  GTCP_print(0x01);
}

/**
Separates all of the screens into four separate screens.

@return none
*/
void GTCP_separateScreens(){
  GTCP_usCommand();
  GTCP_print('w');
  GTCP_print(0x10);
  GTCP_print(0x00);
}

/**
Real-time bit image display.

<ul>
<li> format = 0x81: Monochrome (1-bit) format
<li> format = 0x86: Color 6-bit format
<li> format = 0x8c: Color 12-bit format
<li> format = 0x90: Color 16-bit format
<li> format = 0x98: Color 24-bit format
<li> format = 0xf0: BMP

@param  width  The width of the image to display.
@param  height The height of the image to display.
@param  format The desired image format.
@param  data   The raw data of the image to display.
@return none
*/
void GTCP_drawImage(unsigned width, unsigned height, uint8_t format, char *data){
  //BMP image filesize
  unsigned long int filesize = 0;

  if(height > HEIGHT){
    return;
  }
  GTCP_usCommand();
  GTCP_print('f');
  GTCP_print(0x11);
  GTCP_XY1(width, height);
  GTCP_print(format);
  
  switch(format){
    case 0x81: //Monochrome (1 bit per pixel)
      //Deal with widths that are not a multiple of 8
      while(width % 8){
        width++;
      }
      for(unsigned i = 0; i <= (height * (width / 8)) - 1; i++){
        GTCP_print(data[i]);
      }
      break;
    case 0x86: //6-bit (1 byte per pixel)
      for(unsigned i = 0; i <= (height * width) - 1; i++){
        GTCP_print(data[i]);
      }
      break;
    case 0x8c: //12-bit (2 bytes per pixel)
      for(unsigned i = 0; i <= ((height * width) * 2) - 1; i++){
        GTCP_print(data[i]);
      }
      break;
    case 0x90: //16-bit (2 bytes per pixel)
      for(unsigned i = 0; i <= ((height * width) * 2) - 1; i++){
        GTCP_print(data[i]);
      }
      break;
    case 0x98: //24-bit (3 bytes per pixel)
      for(unsigned i = 0; i <= ((height * width) * 3) - 1; i++){
        GTCP_print(data[i]);
      }
      break;
    case 0xf0: //BMP (entire file)
      filesize = (unsigned long int)data[2] | ((unsigned long int)data[3] << 8) | ((unsigned long int)data[4] << 16) | ((unsigned long int)data[5] << 24);

      for(unsigned long int i = 0; i <= filesize - 1; i++){
        GTCP_print(data[i]);
      }
      break;
  }
}

/**
Draw a pixel on the display.
<ul>
<li> pen = 0x00: Pen color is set to the background color.
<li> pen = 0x01: Pen color is set to the character color.
</ul>

@param  pen Set pen color to character or background color.
@param  x   Pixel position X.
@param  y   Pixel position Y.
@return none
*/
void GTCP_pixelDrawing(uint8_t pen, unsigned x, unsigned y){
  GTCP_usCommand();
  GTCP_print(0x64);
  GTCP_print(0x10);
  GTCP_print(pen);
  GTCP_XY1(x, y);
}

/**
Draw a line or box on the display.
<ul>
<li> mode = 0x00: Line
<li> mode = 0x01: Box
<li> mode = 0x02: Box Fill
</ul>
<ul>
<li> pen = 0x00: Pen color is set to the background color.
<li> pen = 0x01: Pen color is set to the character color.
</ul>

@param  mode   The desired drawing mode.
@param  pen    Set pen color to character or background color.
@param  xStart Line/Box drawing x-axis start position.
@param  yStart Line/Box drawing y-axis start position.
@param  xEnd   Line/Box drawing x-axis end position.
@param  yEnd   Line/Box drawing y-axis end position.
@return none
*/
void GTCP_drawLineBox(uint8_t mode, uint8_t pen, unsigned xStart, unsigned yStart, unsigned xEnd, unsigned yEnd){
  GTCP_usCommand();
  GTCP_print(0x64);
  GTCP_print(0x11);
  GTCP_print(mode);
  GTCP_print(pen);
  GTCP_XY1(xStart, yStart);
  GTCP_XY1(xEnd, yEnd);
}

/**
Software datasheet funtion: Downloaded bit image display
Display an image stored in system memory.
<ul>
<li> memory = 0x00: RAM bit image
<li> memory = 0x01: FROM1 bit image
<li> memory = 0x02: Display memory
<li> memory = 0x10: FROM2 base address 0x0000.0000
<li> memory = 0x11: FROM2 base address 0x0100.0000
<li> memory = 0x12: FROM2 base address 0x0200.0000
<li> memory = 0x13: FROM2 base address 0x0300.0000
<li> memory = 0x14: FROM2 base address 0x0400.0000
<li> memory = 0x15: FROM2 base address 0x0500.0000
<li> memory = 0x16: FROM2 base address 0x0600.0000
<li> memory = 0x17: FROM2 base address 0x0700.0000
<li> memory = 0x18: FROM2 base address 0x0800.0000
<li> memory = 0x19: FROM2 base address 0x0900.0000
<li> memory = 0x1a: FROM2 base address 0x0a00.0000
<li> memory = 0x1b: FROM2 base address 0x0b00.0000
<li> memory = 0x1c: FROM2 base address 0x0c00.0000
<li> memory = 0x1d: FROM2 base address 0x0d00.0000
<li> memory = 0x1e: FROM2 base address 0x0e00.0000
<li> memory = 0x1f: FROM2 base address 0x0f00.0000
</ul>
<ul>
<li> format = 0x81: Monochrome (1-bit) format
<li> format = 0x86: Color 6-bit format
<li> format = 0x8c: Color 12-bit format
<li> format = 0x90: Color 16-bit format
<li> format = 0x91: Color 16-bit high-speed format
<li> format = 0x98: Color 24-bit format
<li> format = 0xf0: BMP file format
</ul>

@param  memory    The type of memory where the image is stored.
@param  address   The memory address of the image.
@param  extension The address extension.
@param  xDefine   The defined width to be displayed of the image.
@param  xSize     The width of the image.
@param  ySize     The height of the image.
@param  format    The desired image format.
@return none
*/
void GTCP_storedBitImageDraw(uint8_t memory, unsigned address, unsigned extension, unsigned xDefine, unsigned xSize, unsigned ySize, uint8_t format){
  GTCP_usCommand();
  GTCP_print(0x66);
  GTCP_print(0x10);
  GTCP_print(memory);
  GTCP_print(address);
  GTCP_print(address >> 8);
  GTCP_print(extension);
  GTCP_print(xDefine);
  GTCP_print(xDefine >> 8);
  GTCP_print(xSize);
  GTCP_print(xSize >> 8);
  GTCP_print(ySize);
  GTCP_print(ySize >> 8);
  GTCP_print(format);
}

/**
Set the character style.
<ul>
<li> style = 0x00: Normal
<li> style = 0x01: Bold
<li> style = 0x02: Shadow
<li> style = 0x03: Bordering
<li> Default: style = 0x00
</ul>

@param  style The desired character style.
@return none
*/
void GTCP_setCharacterStyle(uint8_t style){
  GTCP_usCommand();
  GTCP_print(0x67);
  GTCP_print(0x41);
  GTCP_print(style);
}

/**
Set the character color.
<ul>
<li> 0x00 = No brightness
<li> 0x7f = 50% brightness
<li> 0xff = Maximum brightness
</ul>

@param  red   Red brightness
@param  green Green brightness
@param  blue  Blue brightness
@return none
*/
void GTCP_setCharacterColor(uint8_t red, uint8_t green, uint8_t blue){
  GTCP_usCommand();
  GTCP_print(0x67);
  GTCP_print(0x50);
  GTCP_print(red);
  GTCP_print(green);
  GTCP_print(blue);
}

/**
Set the character background (highlight color) color.
<ul>
<li> 0x00 = No brightness
<li> 0x7f = 50% brightness
<li> 0xff = Maximum brightness
</ul>

@param  red   Red brightness
@param  green Green brightness
@param  blue  Blue brightness
@return none
*/
void GTCP_setBackgroundColor(uint8_t red, uint8_t green, uint8_t blue){
  GTCP_usCommand();
  GTCP_print(0x67);
  GTCP_print(0x51);
  GTCP_print(red);
  GTCP_print(green);
  GTCP_print(blue);
}

/**
Enter user setup mode.

@return none
*/
void GTCP_enterUserSetupMode(){
  GTCP_print(0x1f);
  GTCP_print(0x28);
  GTCP_print(0x65);
  GTCP_print(0x01);
  GTCP_print(0x49);
  GTCP_print(0x4e);
}

/**
End user setup mode.

@return none
*/
void GTCP_endUserSetupMode(){
  GTCP_print(0x1f);
  GTCP_print(0x28);
  GTCP_print(0x65);
  GTCP_print(0x02);
  GTCP_print(0x4f);
  GTCP_print(0x55);
  GTCP_print(0x54);
}

/**
Enable or disable touch data transmission.
<ul>
<li> enable = 0x00: Transmit OFF
<li> enable = 0x01: Transmit ON
<li> Default: enable = 0x00
</ul>

@param  enable Turn touch data ON or OFF
@return none
*/
void GTCP_touchDataEnable(uint8_t enable){
  GTCP_print(0x1f);
  GTCP_print(0x50);
  GTCP_print(0x20);
  GTCP_print(enable);
}

/**
Selects the currently-active touch panel control channel.
<ul>
<li> 0x00 <= channel <= 0x03
</ul>

@param  channel The desired touch control channel.
@return none
*/
void GTCP_touchChannel(uint8_t channel){
  GTCP_print(0x1f);
  GTCP_print(0x50);
  GTCP_print(0x21);
  GTCP_print(channel);
}

/**
Set the desired touch mode.
<ul>
<li> mode = 0x00: Single touch mode
<li> 0x01 <= mode <= 0x0a: Multi-touch mode (up to 10 touches)
<li> Default mode: 0x00
</ul>

@param  mode The desired touch mode.
@return none
*/
void GTCP_touchModeSelect(uint8_t mode){
  GTCP_print(0x1f);
  GTCP_print(0x50);
  GTCP_print(0x01);
  GTCP_print(mode);
}

/**
Set the coordinate touch mode.
<ul>
<li> 0x00 <= channel <= 0x03
</ul>

@param  channel The desired touch data channel.
@return none
*/
void GTCP_coordinatesMode(uint8_t channel){
  GTCP_print(0x1f);
  GTCP_print(0x50);
  GTCP_print(0x10);
  GTCP_print(channel);
  GTCP_print(0x00);
}
/**
Set the switch matrix mode.
<ul>
<li> 0x00 <= channel <= 0x03
<li> 0x01 <= switchesX <= 0x10
<li> 0x01 <= switchesY <= 0x10
<li> 0x01 <= clearanceX <= 0x10
<li> 0x01 <= clearanceY <= 0x10
</ul>

@param  channel    The desired touch data channel.
@param  switchesX  The number of switches on the x-axis.
@param  switchesY  The number of switches on the y-axis.
@param  clearanceX The size of the non-responsive area in-between switches on the x-axis.
@param  clearanceY The size of the non-responsive area in-between switches on the y-axis.
@return none
*/
void GTCP_switchMatrixMode(uint8_t channel, uint8_t switchesX, uint8_t switchesY, uint8_t clearanceX, uint8_t clearanceY){
  GTCP_print(0x1f);
  GTCP_print(0x50);
  GTCP_print(0x10);
  GTCP_print(channel);
  GTCP_print(0x01);
  GTCP_print(switchesX);
  GTCP_print(clearanceX);
  GTCP_print(switchesY);
  GTCP_print(clearanceY);
}

/**
Function to set the GPIO ports on the GTCP unit to be inputs or outputs.

Settings:
<ul>
<li> portSetting = 0x00 (all input) to 0x0f (all output)
<li> The portSetting parameter is bit-wise to the ports present on the module.
</ul>

@param  portSetting The desired port setting, input or output.
@return none
*/
void GTCP_IOPortSetting(uint8_t portSetting){
  GTCP_print(0x1f);
  GTCP_print(0x28);
  GTCP_print(0x70);
  GTCP_print(0x01);
  GTCP_print(0x00);
  GTCP_print(portSetting);
}

/**
Function to set the output value on the GPIO ports.

Settings:
<ul>
<li> 0x00 < output data value < 0x0f
</ul>

@param  portValue The desired value to put on the port.
@return none
*/
void GTCP_IOPortOutput(uint8_t portValue){
  GTCP_print(0x1f);
  GTCP_print(0x28);
  GTCP_print(0x70);
  GTCP_print(0x10);
  GTCP_print(0x00);
  GTCP_print(portValue);
}

/**
Function to request the GTCP module to read the values present on the GPIO ports.

@return none
*/
void GTCP_IOPortInput(){
  GTCP_print(0x1f);
  GTCP_print(0x28);
  GTCP_print(0x70);
  GTCP_print(0x20);
  GTCP_print(0x00);
}

/**
Sets a memory switch to the passed in value.

@param  memorySW The memory SW to be changed.
@param  data     The data to be written to the memory SW.
@return none
*/
void GTCP_MemorySWSetting(uint8_t memorySW, uint8_t data){
  GTCP_print(0x1f);
  GTCP_print(0x28);
  GTCP_print(0x65);
  GTCP_print(0x03);
  GTCP_print(memorySW);
  GTCP_print(data);
}

/**
Set multiple memory switch values.

<ul>
<li> 0x01 <= numSettings <= 0xff
<li> data should be set up like: 
<li> memory SW number [1], setting value [1],
<li> memory SW number [2], setting value [2],
<li> ... memory SW number [n], setting value [n]
</ul>

@param  numSettings The number of SW settings to be set.
@param  data The setting data.
@return none
*/
void GTCP_MemorySWSettingMulti(uint8_t numSettings, uint8_t *data){
  GTCP_print(0x1f);
  GTCP_print(0x28);
  GTCP_print(0x65);
  GTCP_print(0x03);
  GTCP_print(0xff);
  GTCP_print(numSettings);
  for(int i = 0; i < numSettings * 2; i++){
    GTCP_print(data[i]);
  }
}

/**
Send the contents of memory SW data.

The following data is transmitted:
1. Header | 0x28 | 1 byte
2. Identifier 1 | 0x65 | 1 byte
3. Identifier 2 | 0x04 | 1 byte
4. Data | 0x00-0xff | 1 byte / n bytes

@param  switchNum The memory SW to be read.
@return none
*/
void GTCP_MemorySWDataSend(uint8_t switchNum){
  GTCP_print(0x1f);
  GTCP_print(0x28);
  GTCP_print(0x65);
  GTCP_print(0x04);
  GTCP_print(switchNum);
}

/**
Send the contents of memory Sw data.

<ul>
<li> 0x01 <= numReads <= 0xff
</ul>

@param  numReads The number of reads to perform.
@param  data The memory SW numbers to be read.
@return none
*/
void GTCP_MemorySWDataSendMulti(uint8_t numReads, uint8_t *data){
  GTCP_print(0x1f);
  GTCP_print(0x28);
  GTCP_print(0x65);
  GTCP_print(0x04);
  GTCP_print(0xff);
  GTCP_print(numReads);
  for(int i = 0; i < numReads; i++){
    GTCP_print(data[i]);
  }
}

/**
Defines a 16x16 pixel downloaded character (2-byte character) in the code
specified by codeUpper and codeLower.

codeUpper, codeLower: Depends on currently selected language.

<ul>
<li> codeUpper = 0xec && 0x40 <= codeLower <= 0x4f : Japanese - JIS X0208 (SHIFT-JIS)
<li> codeUpper = 0xfe && 0xa1 <= codeLower <= 0xb0 : Korean - KSC5601-87
<li> codeUpper = 0xfe && 0xa1 <= codeLower <= 0xb0 : Simplified Chinese - GB2312-80
<li> codeUpper = 0xfe && 0xa1 <= codeLower <= 0xb0 : Traditional Chinese - Big-5
</ul>

@param  codeUpper Character code, upper byte
@param  codeLower Character code, lower byte
@param  data      Character definition data
@return none
*/
void GTCP_16x16CharacterDefinition(uint8_t codeUpper, uint8_t codeLower, uint8_t *data){
  GTCP_print(0x1f);
  GTCP_print(0x28);
  GTCP_print(0x67);
  GTCP_print(0x10);
  GTCP_print(codeUpper);
  GTCP_print(codeLower);
  for(int i = 0; i < 32; i++){
    GTCP_print(data[i]);
  }
}

/**
Deletes a defined 16x16 download character in code specified by codeUpper and
codeLower.

codeUpper, codeLower: Depends on currently selected language.

<ul>
<li> codeUpper = 0xec && 0x40 <= codeLower <= 0x4f : Japanese - JIS X0208 (SHIFT-JIS)
<li> codeUpper = 0xfe && 0xa1 <= codeLower <= 0xb0 : Korean - KSC5601-87
<li> codeUpper = 0xfe && 0xa1 <= codeLower <= 0xb0 : Simplified Chinese - GB2312-80
<li> codeUpper = 0xfe && 0xa1 <= codeLower <= 0xb0 : Traditional Chinese - Big-5
</ul>

@param  codeUpper Character code, upper byte
@param  codeLower Character code, lower byte
@return none
*/
void GTCP_16x16CharacterDelete(uint8_t codeUpper, uint8_t codeLower){
  GTCP_print(0x1f);
  GTCP_print(0x28);
  GTCP_print(0x67);
  GTCP_print(0x11);
  GTCP_print(codeUpper);
  GTCP_print(codeLower);
}

/**
Defines a 32x32 pixel downloaded character (2-byte character) in character code
specified by codeUpper and codeLower.

<ul>
<li> codeUpper, codeLower: Depends on currently selected language.
<li> codeUpper = 0xec && 0x40 <= codeLower <= 0x4f : Japanese - JIS X0208 (SHIFT-JIS)
</ul>

@param  codeUpper Character code, upper byte
@param  codeLower Character code, lower byte
@param  data      Character definition data.
@return none
*/
void GTCP_32x32CharacterDefinition(uint8_t codeUpper, uint8_t codeLower, uint8_t *data){
  GTCP_print(0x1f);
  GTCP_print(0x28);
  GTCP_print(0x67);
  GTCP_print(0x14);
  GTCP_print(codeUpper);
  GTCP_print(codeLower);
  for(int i = 0; i < 128; i++){
    GTCP_print(data[i]);
  }
}

/**
Delete a defined 32x32 download character in code specified by codeUpper and
codeLower.
This command is invalid if Japanese is not the selected language.

<ul>
<li> codeUpper, codeLower: Depends on currently selected language.
<li> codeUpper = 0xec && 0x40 <= codeLower <= 0x4f : Japanese - JIS X0208 (SHIFT-JIS)
</ul>

@param  codeUpper Character code, upper byte
@param  codeLower Character code, lower byte
@return none
*/
void GTCP_32x32CharacterDelete(uint8_t codeUpper, uint8_t codeLower){
  GTCP_print(0x1f);
  GTCP_print(0x28);
  GTCP_print(0x67);
  GTCP_print(0x15);
  GTCP_print(codeUpper);
  GTCP_print(codeLower);
}

/**
Save the download characters defined in RAM to FROM (RAM -> FROM)

<ul>
<li> fontSize = 0x01: 6x8 pixels
<li> fontSize = 0x02: 8x16 pixels
<li> fontSize = 0x03: 16x16 pixels
<li> fontSize = 0x04: 16x32 pixels
<li> fontSize = 0x05: 32x32 pixels
<li> fontSize = 0x06: 12x24 pixels
</ul>

@param  fontSize The desired character font size to be saved.
@return none
*/
void GTCP_downloadCharacterSave(uint8_t fontSize){
  GTCP_print(0x1f);
  GTCP_print(0x28);
  GTCP_print(0x65);
  GTCP_print(0x11);
  GTCP_print(fontSize);
}

/**
Transfer the downloaded characters saved in FROM to RAM.

<ul>
<li> fontSize = 0x01: 6x8 pixels
<li> fontSize = 0x02: 8x16 pixels
<li> fontSize = 0x03: 16x16 pixels
<li> fontSize = 0x04: 16x32 pixels
<li> fontSize = 0x05: 32x32 pixels
<li> fontSize = 0x06: 12x24 pixels
</ul>

@param  fontSize The desired character font to be restored.
@return none
*/
void GTCP_downloadCharacterRestore(uint8_t fontSize){
  GTCP_print(0x1f);
  GTCP_print(0x28);
  GTCP_print(0x65);
  GTCP_print(0x21);
  GTCP_print(fontSize);
}

/**
Define the user font table for a specified character size.

<ul>
<li> table = 0x01: 6x8 pixel user table
  <li> P(0x80 - 1) ... P(0x80 - 6) ... P(0xff - 6)
  <li> 6 Bytes / font * 128 characters (768 bytes)
<li> table = 0x02: 8x16 pixel user table
  <li> P(0x80 - 1) ... P(0x80 - 16) ... P(0xff - 16)
  <li> 16 Bytes / font * 128 characters (2048 bytes)
<li> table = 0x03: 12x24 pixel user table
  <li> P(0x80 - 1) ... P(0x80 - 36) ... P(0xff - 36)
  <li> 36 Bytes / font * 128 characters (4608 bytes)
<li> table = 0x04: 16x32 pixel user table
  <li> P(0x80 - 1) ... P(0x80 - 64) ... P(0xff - 64)
  <li> 64 Bytes / font * 128 characters (8192 bytes)
</ul>

It is not possible to only define a part of the character code space.
ONLY VALID IN USER SETUP MODE.

@param  table The desired user table font size.
@param  data  The user table data.
@return none
*/
void GTCP_FROMUserFontDefinition(uint8_t table, uint8_t *data){
  GTCP_print(0x1f);
  GTCP_print(0x28);
  GTCP_print(0x65);
  GTCP_print(0x13);
  GTCP_print(table);
  switch(table){
  case 1: //6x8 character table
    for(int i = 0; i <= 768; i++){
      GTCP_print(data[i]);
    }
    break;
  case 2: //8x16 character table
    for(int i = 0; i < 2048; i++){
      GTCP_print(data[i]);
    }
    break;
  case 3: //12x24 character table
    for(int i = 0; i < 4608; i++){
      GTCP_print(data[i]);
    }
    break;
  case 4: //16x32 character table
    for(int i = 0; i < 8192; i++){
      GTCP_print(data[i]);
    }
    break;
  }
}

/**
Define or delete RAM Macro or RAM Program Macro.

<ul>
<li> 0x0000 <= (lengthLower + lengthUpper * 0x0100) <= 0x0400
<li> (lengthLower + lengthUpper * 0x0100) > 0x0000: Supplied data is stored as Macro
<li> (lengthLower + lengthUpper * 0x0100) = 0x0000: Macro is deleted
</ul>

@param  lengthUpper RAM Macro data length, upper byte
@param  lengthLower RAM Macro data length, lower byte
@param  data        RAM Macro data
@return none
*/
void GTCP_RAMMacroDefineDelete(uint8_t lengthUpper, uint8_t lengthLower, uint8_t *data){
  GTCP_print(0x1f);
  GTCP_print(0x3a);
  GTCP_print(lengthLower);
  GTCP_print(lengthUpper);
  int length = ((lengthUpper & 0xff) << 8 | lengthLower & 0xff);
  for(int i = 0; i < length; i++){
      GTCP_print(data[i]);
  }
}

/**
Define or delete FROM Macro or FROM Program Macro.

<ul>
<li> 0x01 <= registrationNum <= 0x04: FROM Macro number 1-4
<li> 0x0000 <= (lengthLower + lengthUpper * 0x0100) <= 0x2000 (if using 4 Macros), 0x8000 (if using 1 Macro)
  <li> (lengthLower + lengthUpper * 0x0100) > 0: Supplied data is stored as a Macro
  <li> (lengthLower + lengthUpper * 0x0100) = 0: Macro is deleted.
<li> 0x00 <= interval <= 0xff
<li> 0x00 <= idleTime <= 0xff
<li> 0x00 <= data <= 0xff
</ul>

@param  registrationNum FROM Macro registration number
@param  lengthUpper     FROM Macro data length, upper byte
@param  lengthLower     FROM Macro data length, lower byte
@param  interval        Display time interval (interval * IntTime)
@param  idleTime        Idle time for macro repetition (idleTime * IntTime)
@param  data            FROM Macro data
@return none
*/
void GTCP_FROMMacroDefineDelete(uint8_t registrationNum, uint8_t lengthUpper, uint8_t lengthLower, uint8_t interval, uint8_t idleTime, uint8_t *data){
  GTCP_print(0x1f);
  GTCP_print(0x28);
  GTCP_print(0x65);
  GTCP_print(0x12);
  GTCP_print(registrationNum);
  GTCP_print(lengthLower);
  GTCP_print(lengthUpper);
  GTCP_print(interval);
  GTCP_print(idleTime);
  int length = ((lengthUpper & 0xff) << 8 | lengthLower & 0xff);
  for(int i = 0; i < length; i++){
      GTCP_print(data[i]);
  }
}

/**
Continuously execute contents of defined Macro 'definitionNum'.

<ul>
<li> 0x00 <= definitionNum <= 0x04, 0x80 <= definitionNum <= 0x84
<li> definitionNum = 0x00: RAM Macro 0
<li> 0x01 <= definitionNum <= 0x04: FROM Macro 1-4
<li> definitionNum = 0x80: RAM Program Macro 0
<li> 0x81 <= definitionNum <= 0x84: FROM Program Macro 1-4
<li> 0x00 <= interval <= 0xff
<li> 0x00 <= idleTime <= 0xff
</ul>

@param  definitionNum Macro processing definition number
@param  interval      Display time interval (interval * IntTime)
@param  idleTime      Idle time for Macro repetition (idleTime * IntTime)
@return none
*/
void GTCP_macroExecution(uint8_t definitionNum, uint8_t interval, uint8_t idleTime){
  GTCP_print(0x1f);
  GTCP_print(0x5e);
  GTCP_print(definitionNum);
  GTCP_print(interval);
  GTCP_print(idleTime);
}

/**
Macro end condition set.

<ul>
<li> endCodeEnable = 0x00: Macro end code disabled
<li> endCodeEnable = 0x01: Macro end code enabled
<li> 0x00 <= endCode <= 0xff
<li> endScreenSetting = 0x00: Clear screen at Macro end
<li> endScreenSetting = 0x01: Do not clear screen at Macro end
</ul>

@param  endCodeEnable    Macro end code Enable/Disable
@param  endCode          Macro end code
@param  endScreenSetting Macro end clear screen setting
@return none
*/
void GTCP_macroEndCondition(uint8_t endCodeEnable, uint8_t endCode, uint8_t endScreenSetting){
  GTCP_print(0x1f);
  GTCP_print(0x28);
  GTCP_print(0x69);
  GTCP_print(0x20);
  GTCP_print(endCodeEnable);
  GTCP_print(endCode);
  GTCP_print(endScreenSetting);
}

/**
Store the supplied data into general-purpose memory / FROM2.

<ul>
<li> memorySelect = 0x10: FROM2 base address 0x0000.0000
<li> memorySelect = 0x11: FROM2 base address 0x0100.0000
<li> memorySelect = 0x12: FROM2 base address 0x0200.0000
<li> memorySelect = 0x13: FROM2 base address 0x0300.0000
<li> memorySelect = 0x14: FROM2 base address 0x0400.0000
<li> memorySelect = 0x15: FROM2 base address 0x0500.0000
<li> memorySelect = 0x16: FROM2 base address 0x0600.0000
<li> memorySelect = 0x17: FROM2 base address 0x0700.0000
<li> memorySelect = 0x18: FROM2 base address 0x0800.0000
<li> memorySelect = 0x19: FROM2 base address 0x0900.0000
<li> memorySelect = 0x1A: FROM2 base address 0x0A00.0000
<li> memorySelect = 0x1B: FROM2 base address 0x0B00.0000
<li> memorySelect = 0x1C: FROM2 base address 0x0C00.0000
<li> memorySelect = 0x1D: FROM2 base address 0x0D00.0000
<li> memorySelect = 0x1E: FROM2 base address 0x0E00.0000
<li> memorySelect = 0x1F: FROM2 base address 0x0F00.0000
<li> 0x000001 <= (sizeLower + sizeUpper * 0x0100 + sizeExtension * 0x00010000) <= 0x00FFFFFF
<li> 0x000000 <= (addressLower + addressUpper * 0x0100 + addressExtension * 0x00010000h) <= 0x00FFFFFF
<li> memorySelect = 0x30 (General-purpose RAM):
<li> 0x000001 <= (sizeLower + sizeUpper * 0x0100 + sizeExtension * 0x00010000) <= 0x00000400
<li> 0x000000 <= (addressLower + addressUpper * 0x0100 + addressExtension * 0x00010000) <= 0x000003FF
<li> memorySelect = 0x31 (General-purpose FROM):
<li> 0x000001h <= (sizeLower + sizeUpper * 0x0100h + sizeExtension * 0x00010000) <= 0x00001000
<li> 0x000000h <= (addressLower + addressUpper * 0x0100 + addressExtension * 0x00010000) <= 0x0000FFFF
<li> 0x00 <= data <= 0xFF
</ul>

@param  sizeUpper        Data size, upper byte
@param  sizeLower        Data size, lower byte
@param  sizeExtension    Data size, extension byte
@param  memorySelect     Desired memory to use.
@param  addressUpper     Memory address, upper byte
@param  addressLower     Memory address, lower byte
@param  addressExtension Memory address, extension byte
@param  data             Data to store
@return none
*/
void GTCP_memoryStore(uint8_t sizeUpper, uint8_t sizeLower, uint8_t sizeExtension, uint8_t memorySelect, uint8_t addressUpper, uint8_t addressLower, uint8_t addressExtension, uint8_t *data){
  GTCP_print(0x1f);
  GTCP_print(0x28);
  GTCP_print(0x65);
  GTCP_print(0x18);
  GTCP_print(sizeLower);
  GTCP_print(sizeUpper);
  GTCP_print(sizeExtension);
  GTCP_print(memorySelect);
  GTCP_print(addressLower);
  GTCP_print(addressUpper);
  GTCP_print(addressExtension);
  int length = ((sizeUpper & 0xff) << 8 | sizeLower & 0xff);
  for(int i = 0; i < length; i++){
      GTCP_print(data[i]);
  }
}

/**
Transfer data between general-purpose memory / FROM2 areas.

<ul>
<li> 0x10 <= destMemory, srcMemory <= 0x17, destMemory, srcMemory = 0x30, 0x31
<li> destMemory, srcMemory = 0x10: FROM2 base address 0x0000.0000
<li> destMemory, srcMemory = 0x11: FROM2 base address 0x0100.0000
<li> destMemory, srcMemory = 0x12: FROM2 base address 0x0200.0000
<li> destMemory, srcMemory = 0x13: FROM2 base address 0x0300.0000
<li> destMemory, srcMemory = 0x14: FROM2 base address 0x0400.0000
<li> destMemory, srcMemory = 0x15: FROM2 base address 0x0500.0000
<li> destMemory, srcMemory = 0x16: FROM2 base address 0x0600.0000
<li> destMemory, srcMemory = 0x17: FROM2 base address 0x0700.0000
<li> destMemory, srcMemory = 0x18: FROM2 base address 0x0800.0000
<li> destMemory, srcMemory = 0x19: FROM2 base address 0x0900.0000
<li> destMemory, srcMemory = 0x1A: FROM2 base address 0x0A00.0000
<li> destMemory, srcMemory = 0x1B: FROM2 base address 0x0B00.0000
<li> destMemory, srcMemory = 0x1C: FROM2 base address 0x0C00.0000
<li> destMemory, srcMemory = 0x1D: FROM2 base address 0x0D00.0000
<li> destMemory, srcMemory = 0x1E: FROM2 base address 0x0E00.0000
<li> destMemory, srcMemory = 0x1F: FROM2 base address 0x0F00.0000
<li> 0x000000 <= (sizeLower + sizeUpper * 0x100 + sizeExtension * 0x10000) <= 0xFFFFFF
<li> 0x000000 <= (destAddressLower + destAddressUpper * 0x100 + destAddressExtension * 0x10000) <= 0xFFFFFF
<li> 0x000000 <= (srcAddressLower + srcAddressUpper * 0x100 + srcAddressExtension * 0x10000) <= 0xFFFFFF
<li> destMemory, srcMemory = 0x30 (General-purpose RAM):
<li> 0x000001 <= (sizeLower + sizeUpper * 0x100 + sizeExtension * 0x10000) <= 0x000400
<li> 0x000000 <= (destAddressLower + destAddressUpper * 0x100 + destAddressExtension * 0x10000) <= 0x0003FF
<li> 0x000000 <= (srcAddressLower + srcAddressUpper * 0x100 + srcAddressExtension * 0x10000) <= 0x0003FF
<li> destMemory, srcMemory = 0x31 (General-purpose FROM):
<li> 0x000001 <= (sizeLower + sizeUpper * 0x100 + sizeExtension * 0x10000) <= 0x001000
<li> 0x000000 <= (destAddressLower + destAddressUpper * 0x100h + destAddressExtension * 0x10000) <= 0x00FFFF
<li> 0x000000 <= (srcAddressLower + srcAddressUpper * 0x100h + srcAddressExtension * 0x10000) <= 0x00FFFF
<li> 0x00 <= data <= 0xFF
</ul>

@param  sizeUpper            Transfer data size, upper byte
@param  sizeLower            Transfer data size, lower byte
@param  sizeExtension        Transfer data size, extension byte
@param  destMemory           Destination memory select
@param  destAddressUpper     Destination address, upper byte
@param  destAddressLower     Destination address, lower byte
@param  destAddressExtension Destination address, extension byte
@param  srcMemory            Source memory select
@param  srcAddressUpper      Source address, upper byte
@param  srcAddressLower      Source address, lower byte
@param  srcAddressExtension  Source address, extension byte
@return none
*/
void GTCP_memoryTransfer(uint8_t sizeUpper, uint8_t sizeLower, uint8_t sizeExtension, uint8_t destMemory, uint8_t destAddressUpper, uint8_t destAddressLower, uint8_t destAddressExtension, uint8_t srcMemory, uint8_t srcAddressUpper, uint8_t srcAddressLower, uint8_t srcAddressExtension){
  GTCP_print(0x1f);
  GTCP_print(0x28);
  GTCP_print(0x65);
  GTCP_print(0x19);
  GTCP_print(sizeLower);
  GTCP_print(sizeUpper);
  GTCP_print(sizeExtension);
  GTCP_print(destMemory);
  GTCP_print(destAddressLower);
  GTCP_print(destAddressUpper);
  GTCP_print(destAddressExtension);
  GTCP_print(srcMemory);
  GTCP_print(srcAddressLower);
  GTCP_print(srcAddressUpper);
  GTCP_print(srcAddressExtension);
}

/**
Send data stored in general-purpose memory.

<ul>
<li> 0x10 <= memorySelect <= 0x17, memorySelect = 0x30, 0x31
<li> memorySelect = 0x10: FROM2 base address 0x0000.0000
<li> memorySelect = 0x11: FROM2 base address 0x0100.0000
<li> memorySelect = 0x12: FROM2 base address 0x0200.0000
<li> memorySelect = 0x13: FROM2 base address 0x0300.0000
<li> memorySelect = 0x14: FROM2 base address 0x0400.0000
<li> memorySelect = 0x15: FROM2 base address 0x0500.0000
<li> memorySelect = 0x16: FROM2 base address 0x0600.0000
<li> memorySelect = 0x17: FROM2 base address 0x0700.0000
<li> memorySelect = 0x18: FROM2 base address 0x0800.0000
<li> memorySelect = 0x19: FROM2 base address 0x0900.0000
<li> memorySelect = 0x1A: FROM2 base address 0x0A00.0000
<li> memorySelect = 0x1B: FROM2 base address 0x0B00.0000
<li> memorySelect = 0x1C: FROM2 base address 0x0C00.0000
<li> memorySelect = 0x1D: FROM2 base address 0x0D00.0000
<li> memorySelect = 0x1E: FROM2 base address 0x0E00.0000
<li> memorySelect = 0x1F: FROM2 base address 0x0F00.0000
<li> 0x000000 <= (addressLower + addressUpper * 0x100 + addressExtension * 0x10000) <= 0xFFFFFF
<li> 0x000000 <= (sizeLower + sizeUpper * 0x100 + sizeExtension * 0x10000) <= 0xFFFFFF
<li> memorySelect = 0x30 (General-purpose RAM):
<li> 0x000001 <= (sizeLower + sizeUpper * 0x100 + sizeExtension * 0x10000) <= 0x000400
<li> 0x000000 <= (addressLower + addressUpper * 0x100 + addressExtension * 0x10000) <= 0x0003FF
<li> memorySelect = 0x31 (General-purpose FROM):
<li> 0x000001 <= (sizeLower + sizeUpper * 0x100 + sizeExtension * 0x10000) <= 0x001000
<li> 0x000000 <= (addressLower + addressUpper * 0x100 + addressExtension * 0x10000) <= 0x00FFFF
</ul>

The following data is transmitted:
1. Header | 0x28 | 1 byte
2. Identifier | 0x65 | 1 byte
3. Identifier | 0x28 | 1 byte
4. Data | 0x00-0xff | n bytes

@param  sizeUpper Data size, upper byte
@param  sizeLower Data size, lower byte
@param  sizeExtension Data size, extension byte
@param  memorySelect Desired memory select
@param  addressUpper Memory address, upper byte
@param  addressLower Memory address, lower byte
@param  addressExtension Memory address, extension byte
@return none
*/
void GTCP_memorySend(uint8_t sizeUpper, uint8_t sizeLower, uint8_t sizeExtension, uint8_t memorySelect, uint8_t addressUpper, uint8_t addressLower, uint8_t addressExtension){
  GTCP_print(0x1f);
  GTCP_print(0x28);
  GTCP_print(0x65);
  GTCP_print(0x28);
  GTCP_print(sizeLower);
  GTCP_print(sizeUpper);
  GTCP_print(sizeExtension);
  GTCP_print(memorySelect);
  GTCP_print(addressLower);
  GTCP_print(addressUpper);
  GTCP_print(addressExtension);
}

/**
Tell the display to send the desired status information.

<ul>
<li> statusType = 0x01: Boot version information (address, length not used)
<li> statusType = 0x02: Firmware version information (address, length not used)
<li> statusType = 0x10: 2-byte character code information (address, length not used)
<li> statusType = 0x11: Language type information (address, length not used)
<li> statusType = 0x20: Memory checksum information
<li> 0x00 <= address <= 0xff: Start address (Effective address = address * 0x10000)
<li> 0x00 <= length <= 0xff: Data length (Effective data length = length * 0x10000)
<li> statusType = 0x30: Product type information (address, length not used)
<li> statusType = 0x40: Display x pixel information (address, length not used)
<li> statusType = 0x41: Display y pixel information (address, length not used)
</ul>

The following data is transmitted:
1. Header | 0x28 | 1 byte
2. Identifier | 0x65 | 1 byte
3. Identifier | 0x40 | 1 byte
4. Data | 0x00-0xff | statusType = 0x01: 4 bytes
                      statusType = 0x02: 4 bytes
                      statusType = 0x10: 15 bytes
                      statusType = 0x11: 15 bytes
                      statusType = 0x20: 4 bytes
                      statusType = 0x30: 15 bytes
                      statusType = 0x40: 3 bytes
                      statusType = 0x41: 3 bytes

@param  statusType The desired module status to read.
@param  address    Checksum address (optional)
@param  address    Length of data (optional)
@return none
*/
void GTCP_displayStatusSend(uint8_t statusType, uint8_t address, uint8_t length){
  GTCP_print(0x1f);
  GTCP_print(0x28);
  GTCP_print(0x65);
  GTCP_print(0x40);
  if(statusType == 0x20){
    GTCP_print(statusType);
    GTCP_print(address);
    GTCP_print(length);
  }else{
    GTCP_print(statusType);
  }
}

/*
Store touch setting package data.
This function is not intended for common use.
Please contact a Noritake sales consultant for further information.

packageNum: 0x01 <= packageNum <= 0x04

@param  packageNum  The desired package number.
@param  data        Touch setting package data.
@return none
*/
void GTCP_touchSettingPackageDataStore(uint8_t packageNum, uint8_t *data){
  if(packageNum > 4){
    packageNum = 4;
  }
  GTCP_print(0x1f);
  GTCP_print(0x28);
  GTCP_print(0x65);
  GTCP_print(0x1c);
  GTCP_print(packageNum);
  for(int i = 0; i < 1024; i++){
    GTCP_print(data[i]);
  }
}

/*
Select the desired touch setting data package to use.
After this command is executed, touch control will start with the selected
touch setting package number.

packageNum: 0x00 <= packageNum <= 0x04
default: packageNum = 0x00

@param  packageNum The desired package number.
@return none
*/
void GTCP_touchSettingPackageSelection(uint8_t packageNum){
  GTCP_print(0x1f);
  GTCP_print(0x4b);
  GTCP_print(0x70);
  GTCP_print(0x10);
  GTCP_print(packageNum);
}