/**
@author  Cody Johnson
@email   cody.ele@noritake.com (Feel free to ask questions)
@website noritake-elec.com
@version v1.1 (ME-N58-1)
@ide     IAR
@brief   I2C library for STM32F4xx and Noritake GT-CP Series Modules

@verbatim
Provide the I2C interface between STM32F4Discovery and Noritake GT-CP modules.
This library uses the STM32F4Discovery library functions from stm32f4xx_i2c.
@endverbatim
*/

#include "GTCP_I2C.h"
#include "GTCP_SPI.h"
#include "stdint.h"
#include <stddef.h>

uint8_t address;

void Delay2(__IO uint32_t time);
extern __IO uint32_t timingDelay;

void Delay2(__IO uint32_t time){
  timingDelay = time;
  while(timingDelay != 0);
}


/**
Initializes I2C communication for STM32F4xx to work with Noritake
GTCP series modules with the I2C daughter board.

@param  GTCPaddress The I2C address for the GTCP module.
@return void
*/
void GTCP_I2C_Init(uint8_t GTCPaddress){
  interface = 2;
  address = GTCPaddress;
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_I2C1, ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOB, ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD, ENABLE);
	
  //Initialize GPIOB Pin 6 (SCL) and 7 (SDA) for alternate function
  GPIO_InitTypeDef GPIOInitAF;
  GPIOInitAF.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7;
  GPIOInitAF.GPIO_Mode = GPIO_Mode_AF;
  GPIOInitAF.GPIO_Speed = GPIO_Speed_2MHz;
  GPIOInitAF.GPIO_OType = GPIO_OType_OD;
  GPIOInitAF.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_Init(GPIOB, &GPIOInitAF);

  //Initialize GPIOB Pin 3 for MBUSY input
  GPIO_InitTypeDef GPIOInit;
  GPIOInit.GPIO_Pin = GPIO_Pin_3;
  GPIOInit.GPIO_Mode = GPIO_Mode_IN;
  GPIOInit.GPIO_Speed = GPIO_Speed_50MHz;
  GPIOInit.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(GPIOB, &GPIOInit);
  
  //Initialize GPIOD PIN 3 for /TRDY input
  GPIO_InitTypeDef GPIODInit;
  GPIODInit.GPIO_Pin = GPIO_Pin_3;
  GPIODInit.GPIO_Mode = GPIO_Mode_IN;
  GPIODInit.GPIO_Speed = GPIO_Speed_50MHz;
  GPIODInit.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(GPIOD, &GPIODInit);
  
  //Change Pins 6 and 7 to their alternate function
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource6, GPIO_AF_I2C1);
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource7, GPIO_AF_I2C1);

  I2C_InitTypeDef I2CInitDef;
  I2CInitDef.I2C_ClockSpeed = 400000;
  I2CInitDef.I2C_Mode = I2C_Mode_I2C;
  I2CInitDef.I2C_DutyCycle = I2C_DutyCycle_2;
  I2CInitDef.I2C_OwnAddress1 = 0x00;
  I2CInitDef.I2C_Ack = I2C_Ack_Enable;
  I2CInitDef.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;
  
  I2C_Cmd(I2C1, ENABLE);
  I2C_Init(I2C1, &I2CInitDef);
}

/**
Starts the I2C communication on STM32F4xx.

@param  direction The master's communication direction.
@return void
*/
int GTCP_I2C_Start(uint8_t direction){
  int result = 0;
  while(I2C_GetFlagStatus(I2C1, I2C_FLAG_BUSY));
  I2C_GenerateSTART(I2C1, ENABLE);
  while(I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT) == ERROR);
  I2C_Send7bitAddress(I2C1, address << 1, direction);

  if(direction == I2C_Direction_Transmitter){
    while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED));
  }
  if(direction == I2C_Direction_Receiver){
    while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_RECEIVER_MODE_SELECTED));
  }
  return result;
}

/**
Reads a single byte from the selected slave device and then sends an
acknowledge to request another byte of data.

@return read data
*/
uint8_t GTCP_I2C_SingleReadAck(){
	uint8_t result;

	GTCP_I2C_Start(I2C_Direction_Receiver);
	I2C_AcknowledgeConfig(I2C1, ENABLE);
	while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_RECEIVED));
	result = I2C_ReceiveData(I2C1);
        GTCP_I2C_Stop();
	return result;
}

/**
Reads a single byte from the selected slave device and then sends an
acknowledge to request another byte of data. This function does not
start the I2C communication. It is meant to be used in succession.

@return read data
*/
uint8_t GTCP_I2C_ReadAck(){
	uint8_t result;
        
	I2C_AcknowledgeConfig(I2C1, ENABLE);
	while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_RECEIVED));
	result = I2C_ReceiveData(I2C1);
	return result;
}

/**
Reads a single byte from the selected slave device and does not
request another byte of data.

@return read data
*/
uint8_t GTCP_I2C_SingleReadNack(){
	uint8_t result;
        
	GTCP_I2C_Start(I2C_Direction_Receiver);
	I2C_AcknowledgeConfig(I2C1, DISABLE);
        I2C_GenerateSTOP(I2C1, ENABLE);
	while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_RECEIVED));
	result = I2C_ReceiveData(I2C1);
	return result;
}

/**
Reads a single byte from the selected slave device and does not
request another byte of data. This function does not start the
I2C communication. It is meant to be used in MultiRead.

@return read data
*/
uint8_t GTCP_I2C_ReadNack(){
	uint8_t result;
        
        while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_RECEIVED));
	result = I2C_ReceiveData(I2C1);
	I2C_AcknowledgeConfig(I2C1, DISABLE);
	
	return result;
}

/**
Reads multiple bytes from the selected slave device. The number
of bytes read is set by parameters.

@param  data    The data storage location for read data.
@param  count   The number of bytes to be read.
@return none
*/
void GTCP_I2C_MultiRead(uint8_t* data, int count){
	GTCP_I2C_Start(I2C_Direction_Receiver);
	for(int i = 0; i < count; i++){
		if(i == (count - 1)){
			data[i] = GTCP_I2C_ReadNack();
		}else{
			data[i] = GTCP_I2C_ReadAck();
		}
	}
}

/**
Reads multiple bytes from the selected slave device.
This function reads bytes based on the /TRDY signal.

@param  data The data storage location for read data.
@return none
*/
char *GTCP_I2C_MultiReadTRDY(){
  int i = 0;
  char *RxDataArray1 = (char *)malloc(20);
  
  GTCP_I2C_Start(I2C_Direction_Receiver);
  while(GPIO_ReadInputDataBit(GPIOD, GPIO_Pin_3) == Bit_SET){ //loop while /TRDY is high
    RxDataArray1[i] = GTCP_I2C_ReadAck();
    if(RxDataArray1[0] == 0x10){
      i++;
    }
  }
  if(i > 0){
    RxDataArray1[i] = GTCP_I2C_ReadNack();
  }
  GTCP_I2C_Stop();
  return RxDataArray1;
}

/**
Writes a byte to the selected slave device.

@param  data Data to be written to the device.
@return none
*/
void GTCP_I2C_Write(uint8_t data){
        GTCP_I2C_Start(I2C_Direction_Transmitter);
	I2C_SendData(I2C1, data);
	while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED));

        GTCP_I2C_Stop();
        while(GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_3) == Bit_SET); //MBUSY check     
}

/**
Writes a string to the selected slave device. The
number of byte transmitted is determined by parameters.

@param  data    The data storage location for read data.
@return none
*/
void GTCP_I2C_MultiWrite(uint8_t* data){
        int i = 0;
        
        while(data[i] != NULL){
          GTCP_I2C_Write(data[i]);
          i++;
        }
}

/**
Stops the I2C communication.

@return none
*/
void GTCP_I2C_Stop(){

	I2C_GenerateSTOP(I2C1, ENABLE);
        
        for(int i = 0; i < 1500; i++)
        {
          
        }
}