/**
@author   Cody Johnson
@email    cody.ele@noritake.com (Feel free to ask questions)
@website  noritake-elec.com
@version  v1.1 (ME-N58-1)
@ide      IAR
@brief    UART library for STM32F4xx and Noritake GT-CP Series Modules

@vertatim
Provide the UART interface between STM32F4Discovery and Noritake GT-CP modules.
This library uses the STM32F4Discovery library functions from stm32f4xx_usart.
@endverbatim
*/

#include "GTCP_USART.h"
#include "stdint.h"
#include <stddef.h>

//define HSE_VALUE 8000000 in preprocessor options!

/**
Initialize UART communication.

@param  baudrate The desired baudrate for UART communication.
@return none
*/
void GTCP_UART_Init(uint32_t baudrate){
  interface = 0;
  GPIO_InitTypeDef GPIO_InitStruct;
  GPIO_InitTypeDef GPIO_InitTypeDefStruct2;
  GPIO_InitTypeDef GPIO_InitTypeDefStruct3;
  USART_InitTypeDef USART_InitStruct;
  
  // Start the required peripheral clocks
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_UART4, ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA | RCC_AHB1Periph_GPIOB, ENABLE);
  
  // Set up UART parameters for UART4.
  USART_InitStruct.USART_BaudRate = baudrate;
  USART_InitStruct.USART_WordLength = USART_WordLength_8b;
  USART_InitStruct.USART_StopBits = USART_StopBits_1;
  USART_InitStruct.USART_Parity = USART_Parity_No;
  USART_InitStruct.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
  USART_InitStruct.USART_Mode = USART_Mode_Tx | USART_Mode_Rx;
  USART_Init(UART4, &USART_InitStruct);
  
  // Initialize the transmit and receive signals.
  // TX = PA0
  // RX = PA1
  GPIO_InitStruct.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1;
  GPIO_InitStruct.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStruct.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_Init(GPIOA, &GPIO_InitStruct);
  
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource0, GPIO_AF_UART4);
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource1, GPIO_AF_UART4);
  
  // MBUSY configuration
  // MBUSY = PB3
  GPIO_InitTypeDefStruct3.GPIO_Pin = GPIO_Pin_3;
  GPIO_InitTypeDefStruct3.GPIO_Mode = GPIO_Mode_IN;
  GPIO_Init(GPIOB, &GPIO_InitTypeDefStruct3);
  
  // HBUSY configuration
  // HBUSY = PB4
  GPIO_InitTypeDefStruct2.GPIO_Pin = GPIO_Pin_4;
  GPIO_InitTypeDefStruct2.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitTypeDefStruct2.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitTypeDefStruct2.GPIO_OType = GPIO_OType_PP;
  GPIO_InitTypeDefStruct2.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(GPIOB, &GPIO_InitTypeDefStruct2);
  GPIO_ResetBits(GPIOB, GPIO_Pin_4);
  
  USART_Cmd(UART4, ENABLE);
}

/**
Write a byte to the display.

@param  data The data to be written.
@return none
*/
void writeUART(uint8_t data){
  while(GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_3) == Bit_SET); // wait for MBUSY to go low
  while(!(UART4->SR & 0x00000040));
  USART_SendData(UART4, data);
  while(GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_3) == Bit_SET); // wait for MBUSY to go low
}

/**
Write an array of bytes to the display.

@param  data The data to be written.
@return none
*/
void printUART(char* data){
  int i = 0;
  while(data[i] != NULL){
    writeUART(data[i]);
    i++;
  }
}

/**
Read a byte from the display.

@return The byte read from the display.
*/
uint8_t readUART(){
  uint8_t data;
  while(!USART_GetFlagStatus(UART4, USART_FLAG_RXNE));
  data = UART4->DR;
  return data;
}

/**
Indicate if a byte has been received from the module.
  
@return 1 if a byte has been received, 0 otherwise.
*/
uint8_t receivedUART(){
  uint8_t result;
  if(USART_GetFlagStatus(UART4, USART_FLAG_RXNE)){
    result = 1;
  } else {
    result = 0;
  }
  return result;
}

/**
Set the HBUSY signal.

@return none
*/
void setHBUSY(){
  GPIO_SetBits(GPIOB, GPIO_Pin_4);
}

/**
Clear the HBUSY signal.

@return none
*/
void clearHBUSY(){
  GPIO_ResetBits(GPIOB, GPIO_Pin_4);
}